import { __read, __spreadArray } from "tslib";
import { PropBindType } from "@cloudbase/cals";
import { autorun, observable, remove, untracked } from "mobx";
import { EventProxy } from './eventProxy';
import { generateDataBinds } from "./adapter/generate";
import { debounce } from "lodash-es";
/**
 * 以 widget id 为 key，只记录同个 widget 的最后一次表达式的结果
 * 这里因为同时只会执行（调试）一个表达式，所以只需保存最后一次执行的结果即可
 */
var eventProxy = new EventProxy();
function debounceFnFactory(fn, wait) {
    var args = [];
    for (var _i = 2; _i < arguments.length; _i++) {
        args[_i - 2] = arguments[_i];
    }
    var id = "".concat(fn.name, ":").concat(JSON.stringify(args));
    debounceFnFactory[id] = debounceFnFactory[id] || debounce(fn.bind.apply(fn, __spreadArray([this], __read(args), false)), wait);
    return debounceFnFactory[id];
}
export var debugExprMap = observable({ node: {} });
export function resetDebugExprMap() {
    // FIXME: widgets 中 for 之下 widget 的 dispose 没有被触发，所以这边通过 remove 来删除使原来的 autorun 不再触发
    remove(debugExprMap, 'node');
    debugExprMap.node = {};
    // window['__debugExprMap'] = debugExprMap
}
export function autorunForDebugExpr(_a) {
    var widgetId = _a.widgetId, subForItems = _a.subForItems, globalCtx = _a.globalCtx, getDataContextFn = _a.getDataContextFn, get$wFn = _a.get$wFn, get$scopeFn = _a.get$scopeFn;
    var forDebugExprDispose = autorun(function () {
        // FIXME: 暂时没考虑低码组件开发模式
        var dataContext = untracked(getDataContextFn);
        var $w = untracked(get$wFn);
        var $scope = untracked(get$scopeFn);
        untracked(function () {
            if (!debugExprMap.node[widgetId]) {
                debugExprMap.node[widgetId] = {
                    expr: '',
                    result: ''
                };
            }
        });
        var expr = debugExprMap.node[widgetId].expr;
        untracked(function () {
            var _a;
            debugExprMap.node[widgetId].result = ''; // 先清空，这样如果下边失败了也不会有错误的值
            var fn = (_a = generateDataBinds(globalCtx, [
                {
                    type: PropBindType.expression,
                    bindDataPath: expr,
                    propertyPath: 'expr',
                },
            ])) === null || _a === void 0 ? void 0 : _a['expr'];
            if (fn) {
                var result = fn(subForItems, undefined, dataContext, $w, $scope);
                debugExprMap.node[widgetId].result = result;
            }
            // 这样对于数组只会拿最后一项
            debounceFnFactory(function (widgetId) { return eventProxy.emit('done_' + widgetId); }, 300, widgetId)();
        });
    });
    return forDebugExprDispose;
}
export function evalExpr(widgetId, expr) {
    return new Promise(function (resolve) {
        if (!debugExprMap.node[widgetId])
            resolve(undefined);
        if (debugExprMap.node[widgetId].expr === expr) {
            // 表达式没变，直接返回结果
            resolve(debugExprMap.node[widgetId].result);
        }
        else {
            eventProxy.once('done_' + widgetId, function () {
                resolve(debugExprMap.node[widgetId].result);
            });
            // 触发 autorun
            debugExprMap.node[widgetId].expr = expr;
        }
    });
}
// window['__testExpr'] = evalExpr
